# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
the page implementation for GUI model based testing
"""

from time import sleep
from time import time
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.wait import POLL_FREQUENCY
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.ui import Select
from qaviton.exceptions import ClickExpectationException
from qaviton.exceptions import ElementPresenceException
from qaviton.drivers.common.webelement import WebElement
from qaviton.crosstest import WebDriver, MobileDriver
from qaviton import settings
from qaviton.utils.helpers import dynamic_delay
from qaviton.crosstest import get_driver, get_drivers


class Page:

    def __init__(self, driver, timeout=None, url=None, platform=None):
        """
        :type driver: WebDriver or MobileDriver
        :type url: str
        :type timeout: int | float
        """
        if timeout is None:
            timeout = settings.page_timeout
        self.timeout = timeout
        self.driver = driver
        self.url = url
        self.platform = platform

    @property
    def title(self):
        return self.driver.title

    @classmethod
    def from_platform(cls, platform, request):
        """:rtype Page"""
        return cls(get_driver(platform, request), platform=platform.platform)

    @classmethod
    def from_platforms(cls, platforms, request):
        """:rtype list[Page]"""
        drivers = get_drivers(platforms, request)
        pages = []
        for i in range(platforms):
            pages.append(cls(drivers[i], platform=platforms[i].platform))
        return pages

    def find(self, locator: tuple, timeout: int = 0, index=0):
        """find element with locator value
        :param locator: locate by method like id and value
        :param timeout: how long to search
        :param index: parameter for special cases
                  where a list of elements is in the locator,
                  the default element to return is elements[0]
        :rtype: WebElement
        """
        return self.driver.find(locator, timeout, index)

    def find_all(self, locator: tuple, timeout: int = 0):
        """find all elements with locator value
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: list[WebElement]"""
        return self.driver.find_all(locator, timeout)

    def try_to_find(self, locator: tuple, timeout=0, index=0):
        """try to find element
        :param index: parameter for special cases
                      where a list of elements is in the locator,
                      the default element to return is elements[0]
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: WebElement | None"""
        return self.driver.try_to_find(locator, timeout, index)

    def try_to_find_all(self, locator: tuple, timeout=0):
        """try to find elements
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: list[WebElement]"""
        return self.driver.try_to_find_all(locator, timeout)

    def get_elements_text(self, locator, timeout=0):
        """ get text from elements
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[str]
        """
        return self.driver.get_elements_text(locator, timeout)

    def find_last_children(self, locator, timeout=0):
        """ get the last elements in the tree from the root element
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[WebElement]
        """
        return self.find(locator, timeout).find_last_children(timeout)

    def find_all_last_children(self, locator, timeout=0):
        """ get the last elements in the tree from the root elements
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[WebElement]
        """
        children_elements = []
        for element in self.find_all(locator, timeout):
            children_elements += element.find_last_children(timeout)
        return children_elements

    def find_children(self, locator, timeout=0):
        """ get the child elements of the parent
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[WebElement] | []
        """
        return self.find(locator, timeout).find_children(timeout)

    def find_all_children(self, locator, timeout=0):
        """ get elements children
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[WebElement]
        """
        children_elements = []
        for element in self.find_all(locator, timeout):
            children_elements += element.find_children(timeout)
        return children_elements

    def zoom(self, percent=200, element=None, steps=50):
        """zoom in/out (might not work for any webbrowser)"""
        return self.driver.zoom(percent=percent, element=element, steps=steps)

    def get_page(self, url=None):
        """works for web, not mobile
        if url is set to None nothing will happen
        """
        if self.url is None:
            if url is None:
                return self
            self.driver.get(url)
            return self
        elif url is None:
            self.driver.get(self.url)
            return self
        return self

    def wait_until_page_loads(self, timeout=None):
        """check that the page has finished loading and all elements are present"""
        if timeout is None:
            timeout = self.timeout
        t = time()
        WebDriverWait(self.driver, timeout).until(
            lambda x: self.driver.execute_script("return document.readyState;") == "complete")
        while True:
            c1 = self.count_all_elements_in_the_page()
            sleep(POLL_FREQUENCY)
            c2 = self.count_all_elements_in_the_page()
            sleep(POLL_FREQUENCY)
            c3 = self.count_all_elements_in_the_page()
            if c1 == c2 == c3:
                break
            elif timeout <= time()-t:
                raise TimeoutException
        return self

    def count_all_elements_in_the_page(self):
        """:rtype: int"""
        return len(self.find_all_elements_in_the_page())

    def find_all_elements_in_the_page(self):
        """:rtype: list[WebElement]"""
        return self.driver.find_elements_by_css_selector("*")

    def click(self, locator, timeout=0, index=0):
        """ click on an element
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :type index: int
        :rtype: WebElement
        """
        return self.driver.click(locator, timeout, index)

    def try_to_click(self, locator, timeout=0, index=0):
        """ try to click on an element
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :type index: int
        :rtype: WebElement | None
        """
        try:
            return self.driver.click(locator, timeout, index)
        except:
            return None

    def click_all(self, locator, timeout=0):
        """ click on all located elements
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[WebElement]
        """
        elements = self.find_all(locator, timeout)
        for element in elements:
            element.click(timeout=timeout)
        return elements

    def drag_and_drop(self, source_locator, target_locator, timeout=0):
        """ drag and drop source element on a target element
        :type source_locator: tuple(str, str | list[WebElement] | WebElement)
        :type target_locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: WebElement
        """
        source = self.find(source_locator, timeout)
        self.driver.drag_and_drop(source, self.find(target_locator, timeout))
        return source

    def drag_to_offset(self, locator, offset_x, offset_y, timeout=0):
        """ drag and drop source element on a (x,y) offset
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type offset_x: int
        :type offset_y: int
        :type timeout: int
        :rtype: WebElement
        """
        return self.driver.drag_to_offset(self.find(locator, timeout), offset_x, offset_y)

    def hover(self, locator, timeout=0):
        """ hover/move cursor to element
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: WebElement
        """
        return self.driver.hover(self.find(locator, timeout))

    def hover_and_click(self, locator, timeout=0):
        """ hover/move cursor to element and click
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: WebElement
        """
        return self.driver.hover_and_click(self.find(locator, timeout))

    def scroll_element_into_view(self, locator, timeout=0, retries=5):
        """ selenium only: scrolling function to scroll until element is visible (whole window scroll)
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :param retries: integer bigger or equal to 0
        :rtype: WebElement
        """
        return self.driver.scroll_element_into_view(self.driver.find(locator, timeout), retries)

    def clear(self, locator, timeout=0):
        return self.find(locator, timeout).clear()

    def send(self, locator, keys, timeout=0):
        """ click on element, clear element text, send keys to element.
        :type locator: (tuple(str, str | list[WebElement] | WebElement)
        :type keys: str
        :type timeout: float | int
        :rtype: WebElement
        """
        return self.click(locator, timeout).clear().send_keys(keys)

    def send_chars(self, locator, chars, timeout=0):
        """ click on element, clear element text, send individual characters to element.
        this function is useful in flacky applications, where the send keys function can cause issues.
        :type locator: (tuple(str, str | list[WebElement] | WebElement)
        :type chars: str
        :type timeout: float | int
        :rtype: WebElement
        """
        return self.click(locator, timeout).clear().send_chars(chars)

    def send_all(self, locator, keys, timeout=0):
        """ send text to multiple elements
        :type locator: (tuple(str, str | list[WebElement] | WebElement)
        :type keys: str
        :type timeout: float | int
        :rtype: WebElement
        """
        elements = self.find_all(locator, timeout)
        for element in elements:
            element.click(timeout=timeout).clear().send_keys(keys)
        return elements

    def select(self, locator, choice=None, timeout=0, index=None):
        select = Select(self.driver.find(locator, timeout))
        if choice is not None:
            select.select_by_value(choice)
        elif index is not None:
            select.select_by_index(index)
        return select

    def select_from_dropdown(self, locator, choice, timeout=0):
        """ select a choice from a drop down list
        :type locator: (tuple(str, str | list[WebElement] | WebElement)
        :type choice: str
        :type timeout: float | int
        :rtype: WebElement
        """
        elements = self.find_all_last_children(self.find_all(locator, timeout))
        element = self.try_to_find(locator, timeout)
        element.send_keys(choice)
        for element in elements:
            if element.text == choice:
                element.click()
                return element

    def multi_select_from_dropdown(self, locator_open_list, locator_list, choice, timeout=0):
        """ select a choice from a multi drop down list
        :type locator_open_list: tuple(str, str | list[WebElement] | WebElement)
        :type locator_list: tuple(str, str | list[WebElement] | WebElement)
        :type choice: str
        :type timeout: float | int
        :rtype: WebElement
        """
        element_to_open_or_close_list = self.click(locator_open_list, timeout)
        elements = self.find_all_last_children(self.find_all(locator_list, timeout))
        for element in elements:
            if element.text == choice:
                element.click(timeout=timeout)
                element_to_open_or_close_list.click(timeout=timeout)
                return element

    def replace_url(self, pattern, replace):
        """ replace url pattern
        :type pattern: str
        :type replace: str
        """
        self.driver.get(self.driver.current_url.replace(pattern, replace))

    def try_to_click_until_element_is_created(self, locator_to_click, locator_of_created_element,
                                              retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ try to click on element, check if condition element is created, if not, click again.
        return None for success or Exception for failure,
        if an exception occurred in the last try, it will be returned.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_of_created_element: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool | Exception
        """
        initial_results = len(self.try_to_find_all(locator_of_created_element))

        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_of_created_element, timeout)) > initial_results:
                    return
                elif i == retries:
                    return ClickExpectationException("element creation failed")
            except Exception as e:
                if i == retries:
                    return e
            dynamic_delay(t, delay)

    def click_until_element_is_created(self, locator_to_click, locator_of_created_element,
                                       retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ click on element, check if condition element is created, if not, click again.
        return None for success or raise Exception for failure.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_of_created_element: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool
        """
        initial_results = len(self.try_to_find_all(locator_of_created_element))

        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_of_created_element, timeout)) > initial_results:
                    return
                elif i == retries:
                    raise ClickExpectationException("element creation failed")
            except Exception as e:
                if i == retries:
                    raise ClickExpectationException("element creation failed") from e
            dynamic_delay(t, delay)

    def try_to_click_until_condition_as_expected(self, locator_to_click, locator_condition,
                                                 condition_expected_count_results,
                                                 retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ try to click on element, check if:
        locator_condition count is equal to condition_expected_count_results, if not, click again.
        return None for success or Exception for failure,
        if an exception occurred in the last try, it will be returned.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_condition: tuple(str, str | list[WebElement] | WebElement)
        :type condition_expected_count_results: int
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool | Exception
        """
        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_condition, timeout)) == condition_expected_count_results:
                    return
                elif i == retries:
                    return ClickExpectationException("elements count expectation failed")
            except Exception as e:
                if i == retries:
                    return e
            dynamic_delay(t, delay)

    def click_until_condition_as_expected(self, locator_to_click, locator_condition,
                                          condition_expected_count_results,
                                          retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ click on element, check if:
        locator_condition count is equal to condition_expected_count_results, if not, click again.
        return None for success or raise Exception for failure.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_condition: tuple(str, str | list[WebElement] | WebElement)
        :type condition_expected_count_results: int
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool
        """
        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_condition, timeout)) == condition_expected_count_results:
                    return
                elif i == retries:
                    raise ClickExpectationException("elements count expectation failed")
            except Exception as e:
                if i == retries:
                    raise ClickExpectationException("elements count expectation failed") from e
            dynamic_delay(t, delay)

    def try_to_click_until_element_is_deleted(self, locator_to_click, locator_of_deleted_element=None,
                                              retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ try to click on element, check if:
        locator_of_deleted_element has been deleted, if its not deleted retry!
        return None for success or return Exception for failure.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_of_deleted_element: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool | Exception
        """
        if locator_of_deleted_element is None:
            locator_of_deleted_element = locator_to_click

        initial_results = len(self.try_to_find_all(locator_of_deleted_element))

        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_of_deleted_element, timeout)) < initial_results:
                    return
                elif i == retries:
                    return ClickExpectationException("element deletion failed")
            except Exception as e:
                if i == retries:
                    return e
            dynamic_delay(t, delay)

    def click_until_element_is_deleted(self, locator_to_click, locator_of_deleted_element=None,
                                       retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ click on element, check if:
        locator_of_deleted_element has been deleted, if its not deleted retry!
        return None for success or raise Exception for failure.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_of_deleted_element: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool
        """
        if locator_of_deleted_element is None:
            locator_of_deleted_element = locator_to_click

        initial_results = len(self.try_to_find_all(locator_of_deleted_element))

        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.try_to_find_all(locator_of_deleted_element, timeout)) < initial_results:
                    return
                elif i == retries:
                    raise ClickExpectationException("element deletion failed")
            except Exception as e:
                if i == retries:
                    raise ClickExpectationException("element deletion failed") from e
            dynamic_delay(t, delay)

    def try_to_confirm_element_is_deleted(self, locator, expected_count_results=0,
                                          retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ try to find element, confirm element has been deleted with expected_count_results,
        check if element exist... if its not deleted retry!
        return None for success or return Exception for failure.

        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type expected_count_results: int
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool | Exception
        """
        for i in range(1 + retries):
            t = time()
            try:
                if len(self.try_to_find_all(locator, timeout)) <= expected_count_results:
                    return
                elif i == retries:
                    return ElementPresenceException("element should not exist")
            except Exception as e:
                if i == retries:
                    return e
            dynamic_delay(t, delay)

    def confirm_element_is_deleted(self, locator, expected_count_results=0,
                                   retries=7, timeout=2, delay=POLL_FREQUENCY):
        """ try to find element, confirm element has been deleted with expected_count_results,
        check if element exist... if its not deleted retry!
        return None for success or raise Exception for failure.

        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type expected_count_results: int
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool
        """
        for i in range(1 + retries):
            t = time()
            try:
                if len(self.try_to_find_all(locator, timeout)) <= expected_count_results:
                    return
                elif i == retries:
                    raise ElementPresenceException("element should not exist")
            except Exception as e:
                if i == retries:
                    raise ElementPresenceException("element should not exist") from e
            dynamic_delay(t, delay)

    def refresh_page_until_element_is_found(self, locator, timeout=30, retries=2):
        """ let's say you landed on an unexpected page/content
        and you want to refresh and assert you have the correct content (really weird scenario)
        this method will search for an element, if the search result is 0 it will refresh the page and retry.

        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :type retries: int
        :rtype: list[WebElement]
        """
        for i in range(1+retries):
            elements = self.try_to_find_all(locator, timeout)
            if len(elements) == 0:
                self.driver.refresh()
                try:
                    self.wait_until_page_loads()
                except Exception as e:
                    if i == retries:
                        raise e
            else:
                return elements
        raise NoSuchElementException("elements are not present after page refresh")

    def click_until_url_changes(self, locator, retries=5, timeout=2, delay=POLL_FREQUENCY):
        """ click on element, check if url changed, if not, click again
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: int
        :type delay: int
        :rtype: list[WebElement]
        """
        url = self.driver.current_url
        for i in range(1+retries):
            t = time()
            try:
                self.click(locator, timeout)
            except Exception as e:
                if i == retries:
                    raise ClickExpectationException("url did not change") from e
            if url != self.driver.current_url:
                return
            elif i == retries:
                raise ClickExpectationException("url did not change")
            dynamic_delay(t, delay)

    def not_clickable(self, locator, timeout=0):
        """ use this with assert, example:
                assert page.not_clickable(locator.ID("dis-button"), timeout=3)

        some elements might be click-able but the click action does nothing,
        in that case use something like click_do_nothing(locator_to_click, locator_expectation)
        where you can check if some event happened.

        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: bool
        """
        try:
            self.click(locator, timeout=timeout)
        except:
            return True
        return False

    def click_do_nothing(self, locator_to_click, locator_expectation=None,
                         retries=1, timeout=2, delay=POLL_FREQUENCY):
        """ use this with assert, example:
                assert page.click_do_nothing(locator.ID("dis-button"), timeout=3)

        check if some event(locator_expectation) happened after trying to click.

        :type locator_to_click: tuple(str, str | list[WebElement] | WebElement)
        :type locator_expectation: tuple(str, str | list[WebElement] | WebElement)
        :type retries: int
        :type timeout: float | int
        :type delay: float | int
        :rtype: bool
        """
        if locator_expectation is None:
            locator_expectation = ('css', '*')

        initial_results = len(self.try_to_find_all(locator_expectation))

        for i in range(1+retries):
            t = time()
            try:
                self.click(locator_to_click, timeout)
                if len(self.find_all(locator_expectation, timeout)) != initial_results:
                    return False
            except:
                return True
            dynamic_delay(t, delay)
        return True

    def no_such_element(self, locator, expected_elements=0, timeout=0):
        """ use this with assert, example:
                assert page.no_such_element(locator.ID("no-el"), timeout=3)

        check if some event(locator_expectation) happened after trying to click.
        search for element, expect not to find it
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type expected_elements: int
        :type timeout: int
        :rtype: bool
        """
        if len(self.try_to_find_all(locator, timeout)) > expected_elements:
            return False
        return True



import sys
import inspect
import time


def timestamp():
    return str(int(time.time() * 1000))


def get_python_version():
    return sys.version_info


def funcname():
    return inspect.stack()[1][3]


def classname(frame=None):
    if frame is None:
        frame = inspect.stack()[1][0]
    args, _, _, value_dict = inspect.getargvalues(frame)
    # now lets check if the 1st parameter in the frame is named 'self'
    if len(args) and args[0] == 'self':
        # in that case self will be referenced in value_dict
        instance = value_dict.get('self', None)
        if instance:
            # return its class
            return getattr(instance, '__class__', None).__name__
    return None


class DynamicClass:
    """ do not touch this class!
        it is intended for dynamic objects
    """
    def __init__(self):
        pass


def list_to_object(keys, values):
    """
    create object in runtime.
    keys & values are lists/tuples with equal length
    :param keys: list
    :param values: list
    :return: dynamic_object
    """
    obj = DynamicClass()
    for i in range(len(keys)):
        setattr(obj, keys[i], values[i])
    return obj


def dict_to_object(dictionary):
    """
    parse a dictionary and return a dynamic object
    :param dictionary: dict
    :return: dynamic_object
    """
    obj = DynamicClass()
    for key in dictionary:
        setattr(obj, key, dictionary[key])
    return obj


def object_to_dict(object):
    """
    parse an object and return a dictionary
    :param object: object
    :return: dict
    """
    dictionary = {}
    obj_dict = object.__dict__
    for key in obj_dict:
        if not key.startswith('__'):
            dictionary[key] = obj_dict[key]
    return dictionary


def pop_by_name(items, name):
    """pop list item by its value"""
    if name in items:
        items.pop(name)
    return name


def swap(items, a, b):
    """swap between list/dict items values"""
    items[a], items[b] = items[b], items[a]
    return items


def dynamic_delay(t, max_delay):
    """ sleep dynamically by measuring the time passed
    since t started and sleeping the
    remaining of the delay

    :param t: time.time()
    :param max_delay: in seconds
    """
    t = time.time() - t
    if t < max_delay:
        time.sleep(max_delay - t)


def list_diff(list1, list2):
    """get the outer-section between lists
    example:
        a = ['a','b','c']
        b = ['a','b']
        list_diff(a, b)
        >['c']
    """
    s = set(list2)
    return [x for x in list1 if x not in s]


def string_to_ascii(string: str):
    """return string of ascii characters with _ as seperator"""
    return '_'.join(str(ord(c)) for c in string)


def ascii_to_string(string: str):
    """return ascii characters with _ as seperator to string"""
    return ''.join(chr(int(i)) for i in string.split('_'))



import pytest
from qaviton.utils.differ import asserts
from qaviton.exceptions import DiffException


def test_differ():
    asserts.elements_are_equal(1, 1, 1, 1, 1)
    asserts.elements_are_equal("1", "1")
    with pytest.raises(DiffException):
        asserts.elements_are_equal("1", "2")



from qaviton import crosstest
from tests.config.private import hub
from tests.config.navigation_platforms import sessionTimeout, app, screenResolution


# create cross-platform testing object
platforms = crosstest.Platforms()
platforms.web.command_executor = hub


# add chrome browser support
platforms.web({
    'browserName': "chrome",
    'version': "",
    'platform': "ANY",
    'app': app,
    'screenResolution': screenResolution,
    'sessionTimeout': sessionTimeout,
    'enableVNC': True,
    'enableVideo': True,
    'name': "{}",
    'videoName': "{}.mp4",
    'logName': "{}.log"})




import os
from qaviton.utils import filer
from qaviton.utils import path
from qaviton.utils.operating_system import s
from qaviton.version import __version__

cwd = os.getcwd()
examples = path.of(__file__)('examples')


def initial_msg(f):
    def dec(*args, **kwargs):
        print("""
            QAVITON VERSION {}
            creating qaviton framework and test examples

            """.format(__version__))
        f(*args, **kwargs)

    return dec


def install_is_done(tests_dir):
    print("""


            @@@@@@@@@@@@@@@@@@@@@
            @ installation done @
            @@@@@@@@@@@@@@@@@@@@@

        # use pip install, uninstall & freeze 
        # to manage your dependencies:
        (venv) path/to/project> pip freeze > requirements-test.txt

        # to install your requirements on a new machine(consider using git):
        (venv) path/to/project> pip install -r requirements-test.txt

            * your testing framework is done!
            * start testing like a boss ⚛
            *      ______________                                                                                                  
            *    /   __________   \           ______                                                                           
            *   /  /            \  \         / ____ \                                                                      
            *  /  /   \      /   \  \       / /    \ \    __        __   _   ___________    _______    _     _                                             
            *  | |   O \    / O   | |      / |______| \   \ \      / /  |_| |____   ____|  / _____ \  | \   | |                            
            *  | |                | |     |  ________  |   \ \    / /   |-|      | |      | |     | | |  \  | |                        
            *  \  \  \________/  /   \    | |        | |    \ \  / /    | |      | |      | |     | | | | \ | |            
            *   \  \____________/  /\ \_  | |        | |     \ \/ /     | |      | |      | |_____| | | |\ \| |            
            *    \________________/  \__| |_|        |_|      \__/      |_|      |_|       \_______/  |_| \___|            

        """)


def add_readme(tests_dir):
    with open(cwd + s + tests_dir + s + 'README.rst', 'w+') as f:
        f.write("this should be changed to a custom README file for your project\n"
                "you have a nice starting point from here.\n"
                "\n"
                "requirements\n"
                "------------\n"
                "python 3.7 and above\n"
                "pytest latest\n"
                "\n"
                "\n"
                "testing examples\n"
                "----------------\n"
                "checkout under execute_tests/end_to_end_tests to see testing examples.\n"
                "\n"
                "\n"
                "model-based examples\n"
                "--------------------\n"
                "check out the pages directory for page model examples\n"
                "and services/app for a model-based-app service for testing.\n"
                "\n"
                "\n"
                "conftest & pytest.ini\n"
                "---------------------\n"
                "look at the conftest.py to see how to add model-based fixtures\n"
                "you can set parallel testing & reporting with pytest.ini file\n"
                "\n"
                "\n"
                "setup your hub\n"
                "--------------\n"
                "check out in the data dir to set your secret user key and remote hub\n"
                "and customize your supported platforms under the data/supported_platfoms.py file.\n"
                "\n"
                "\n"
                "local hub\n"
                "---------\n"
                "install docker:\n"
                "https://docs.docker.com/install/\n"
                "\n"
                "install selenoid:\n"
                "go to option 2 to install with docker\n"
                "https://github.com/aerokube/selenoid/blob/master/docs/quick-start-guide.adoc\n"
                "\n"
                "go to your secret file and change your hub url to local host:\n"
                "/project/tests/data/secret.py\n"
                "hub='http://localhost:4444/wd/hub'\n"
                "\n"
                "\n"
                "run tests examples\n"
                "------------------\n"
                "cd to your project(in my case it's called myapp and my tests are under tests) and simply\n"
                "(env) C:\\Users\\user\\PycharmProjects\\myapp>python -m pytest tests\n"
                "\n"
                "or run with pycharm:\n"
                "https://www.jetbrains.com/pycharm/download/#section=windows\n"
                "https://www.jetbrains.com/help/pycharm/pytest.html")


def add_gitignore(tests_dir):
    if not filer.os.path.exists(cwd + s + '.gitignore'):
        with open(cwd + s + tests_dir + s + '.gitignore', 'w+') as f:
            f.write("# Byte-compiled / optimized / DLL files\n"
                    "__pycache__/\n"
                    "*.py[cod]\n"
                    "*$py.class\n"
                    "\n"
                    "# C extensions\n"
                    "*.so\n"
                    "\n"
                    "# Distribution / packaging\n"
                    ".Python\n"
                    "build/\n"
                    "develop-eggs/\n"
                    "dist/\n"
                    "downloads/\n"
                    "eggs/\n"
                    ".eggs/\n"
                    "lib/\n"
                    "lib64/\n"
                    "parts/\n"
                    "sdist/\n"
                    "var/\n"
                    "wheels/\n"
                    "*.egg-info/\n"
                    ".installed.cfg\n"
                    "*.egg\n"
                    "MANIFEST\n"
                    "\n"
                    "# PyInstaller\n"
                    "#  Usually these files are written by a python script from a template\n"
                    "#  before PyInstaller builds the exe, so as to inject date/other infos into it.\n"
                    "*.manifest\n"
                    "*.spec\n"
                    "\n"
                    "# Installer logs\n"
                    "pip-log.txt\n"
                    "pip-delete-this-directory.txt\n"
                    "\n"
                    "# Unit test / coverage reports\n"
                    "htmlcov/\n"
                    ".tox/\n"
                    ".coverage\n"
                    ".coverage.*\n"
                    ".cache\n"
                    "nosetests.xml\n"
                    "coverage.xml\n"
                    "*.cover\n"
                    ".hypothesis/\n"
                    ".pytest_cache/\n"
                    "\n"
                    "# Translations\n"
                    "*.mo\n"
                    "*.pot\n"
                    "\n"
                    "# Django stuff:\n"
                    "*.log\n"
                    "\n"
                    "# Scrapy stuff:\n"
                    ".scrapy\n"
                    "\n"
                    "# Sphinx documentation\n"
                    "docs/_build/\n"
                    "\n"
                    "# PyBuilder\n"
                    "target/\n"
                    "\n"
                    "# Jupyter Notebook\n"
                    ".ipynb_checkpoints\n"
                    "\n"
                    "# pyenv\n"
                    ".python-version\n"
                    "\n"
                    "# celery beat schedule file\n"
                    "celerybeat-schedule\n"
                    "\n"
                    "# SageMath parsed files\n"
                    "*.sage.py\n"
                    "\n"
                    "# Environments\n"
                    ".env\n"
                    ".venv\n"
                    "env/\n"
                    "venv/\n"
                    "ENV/\n"
                    "env.bak/\n"
                    "venv.bak/\n"
                    "\n"
                    "# Spyder project settings\n"
                    ".spyderproject\n"
                    ".spyproject\n"
                    "\n"
                    "# Rope project settings\n"
                    ".ropeproject\n"
                    "\n"
                    "# mkdocs documentation\n"
                    "/site\n"
                    "\n"
                    "# mypy\n"
                    ".mypy_cache/\n"
                    "\n"
                    "# private\n"
                    "*secret*")


def add_pytest_ini(tests_dir):
    with open(cwd + s + tests_dir + s + 'pytest.ini', 'w+') as f:
        f.write("[pytest]\n"
                ";addopts = -n 3\n"
                ";addopts = --html=report.html\n"
                ";addopts = --junitxml=\path\\to\\reports\n"
                ";addopts = --collect-only\n"
                ";addopts = --cov=your_app")


def add_requirements(tests_dir):
    if not filer.os.path.exists(cwd + s + 'requirements-test.txt'):
        open(cwd + s + 'requirements-test.txt', 'w+').close()
        os.system('pip freeze > requirements-test.txt')


# TODO: add more content for different frameworks
@initial_msg
def framework(frameworks, tests_dir, params):
    if '--example' in params:
        filer.copy_directory(examples + s + 'simple_web', cwd + s + tests_dir)
        add_readme(tests_dir)

    else:
        filer.copy_directory(examples + s + 'new_project', cwd + s + tests_dir)

    if tests_dir != 'tests':
        filer.find_replace(cwd + s + tests_dir, 'from tests.', 'from ' + tests_dir + '.', "*.py")

    add_pytest_ini(tests_dir)
    add_gitignore(tests_dir)
    add_requirements(tests_dir)
    install_is_done(tests_dir)



from tests.pages.components.page import Page
from tests.config.locators import locator
from tests.pages.components.google_search_bar import GoogleSearchBar
from selenium.webdriver.common.keys import Keys


class GoogleSearchPage(Page):
    def __init__(self, driver):
        Page.__init__(self, driver)
        self.search_bar = GoogleSearchBar(driver)

    def google_search_result(self, text):
        return self.find(locator.text(text))

    def go_to(self, search):
        self.search_bar.button().send(search).send_keys(Keys.ENTER)
        self.wait_until_page_loads()
        try:
            self.google_search_result(search).click()
            self.wait_until_page_loads()
        except Exception as e:
            import traceback
            print(traceback.format_exc())
            text = []
            for span in self.find_all(locator.xpath('//span[@dir="ltr"]')):
                if search in span.text or search == span.text:
                    span.click_at()
                    span.try_to_click()
                    self.wait_until_page_loads()
                    assert self.no_such_element(locator.xpath('//span[@dir="ltr"]'))
                    return
                text.append(span.text)
            raise Exception('{} - text not found: {}'.format(search, text)) from e
        
    def navigate_to_LinkedinHomePage(self, weight=5, *args, **kwargs):
        try:
            self.go_to('LinkedIn: Log In or Sign Up')
        except:
            self.get_page("https://www.linkedin.com/")
            self.wait_until_page_loads()

    def navigate_to_YnetHomePage(self, weight=5, *args, **kwargs):
        try:
            self.go_to('ynet - חדשות, כלכלה, ספורט, בריאות')
        except:
            self.get_page("https://www.ynet.co.il")
            self.wait_until_page_loads()

    def navigate_to_GoogleSearchPage(self, weight=5, *args, **kwargs):
        pass



"""
use this script to add your module to the python path.
this way tou can run your module/package locally from anywhere

example_1:
    description:
        put the script in the directory you want to add
        to the path and run the script.

    shell commands:
        python add_python_module_to_path.py

example_2:
    description:
        call the script and use the module_path
        parameter to point it to the module path

    shell commands:
        python add_python_module_to_path.py module_path /path/to/module/directory
"""
import os
import sys
from distutils.sysconfig import get_python_lib


if os.name == 'nt':
    s = '\\'
else:
    s = '/'


def get_external_arguments():
    args = sys.argv
    module_path = None
    if len(args) > 2:
        for i in range(1, len(args)):
            if args[i] == 'module_path':
                if len(args) > i + 1:
                    module_path = args[i+1]
    return module_path


def get_module_default_path():
    return os.path.dirname(os.path.abspath(__file__))


def get_project_name(module_directory):
    return module_directory.split(s)[-1]


def create_new_site_package(project_name, module_directory):
    path = '{}{}{}.pth'.format(get_python_lib(), s, project_name)
    if not os.path.exists(path):
        path_file = open(path, 'w')
        path_file.write(module_directory)
        path_file.close()
    else:
        raise Exception("path file: {} exist".format(path))


def add_module_to_site_packages(module_path=None):
    if module_path is None:
        module_path = get_module_default_path()
    project_name = get_project_name(module_path)
    create_new_site_package(project_name, module_path)


if __name__ == "__main__":
    add_module_to_site_packages(get_external_arguments())



# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
the WebFunctions class includes functions
that are common to both driver & element classes of any kind.
this common functionality will provide the best flexibility
possible for automating simple and complex scenarios
"""

from selenium.webdriver.support.ui import WebDriverWait
from qaviton.locator import ByExtension
from qaviton.locator import Locator


class presence_of_element_located(object):
    """ An expectation for checking that an element is present on the DOM
    of a page. This does not necessarily mean that the element is visible.
    webLocator - used to find the element
    returns the WebElement once it is located
    :rtype: WebElement
    """

    def __init__(self, locator, index=0):
        self.locator = locator
        self.index = index

    def __call__(self, driver):
        return WebFunctions.find(driver, self.locator, self.index)


class presence_of_all_elements_located(object):
    """ An expectation for checking that there is at least one element present
    on a web page.
    webLocator is used to find the element
    returns the list of WebElements once they are located
    :rtype: list[WebElement]
    """
    def __init__(self, locator):
        self.locator = locator

    def __call__(self, driver):
        return WebFunctions.find_all(driver, self.locator)


class WebFunctions:
    """this class is added to ~( webdriver & webelement )~
    to add same functionality to both driver & element
    """

    def find(self, locator: tuple, timeout: int=0, index=0):
        """find element with locator value
        :param locator: locate by method like id and value
        :param timeout: how long to search
        :param index: parameter for special cases
                  where a list of elements is in the locator,
                  the default element to return is elements[0]
        :rtype: WebElement
        """

        # use this locate method if timeout is 0
        def fast(locate):
            return self.find_element(*locate)

        # use this locate method if timeout > 0
        def slow(locate):
            return WebDriverWait(self, timeout).until(presence_of_element_located(locate))

        # redundancy
        if isinstance(locator[0], tuple) or isinstance(locator[0], list):
            element = None
            for i in range(len(locator)):
                try:
                    element = self.find(locator[i], timeout=timeout, index=index)
                    break
                except Exception as e:
                    if len(locator) == i + 1:
                        raise e
            return element

        # find with index
        if locator[0] == ByExtension.INDEX:
            return self.find_all(locator[2])[locator[1]]

        # find with any other strategy
        by, value = Locator.any(locator)

        # if element is already found
        if by == ByExtension.ELEMENTS:
            return value[index]
        elif by == ByExtension.ELEMENT:
            return value

        # choose finding method
        if timeout > 0:
            get = slow
        else:
            get = fast

        # find element
        return get((by, value))

    def find_all(self, locator: tuple, timeout: int=0):
        """find all elements with locator value
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: list[WebElement]"""

        # use this locate method if timeout is 0
        def fast(locate):
            return self.find_elements(*locate)

        # use this locate method if timeout > 0
        def slow(locate):
            return WebDriverWait(self, timeout).until(presence_of_all_elements_located(locate))

        # redundancy
        if isinstance(locator[0], tuple) or isinstance(locator[0], list):
            elements = None
            for i in range(len(locator)):
                try:
                    elements = self.find_all(locator[i], timeout=timeout)
                    break
                except Exception as e:
                    if len(locator) == i+1:
                        raise e
            return elements

        # find with index
        if locator[0] == ByExtension.INDEX:
            return self.find_all(locator[2])[locator[1]]

        # find with any other strategy
        by, value = Locator.any(locator)

        # if element is already found
        if by == ByExtension.ELEMENTS:
            return value
        elif by == ByExtension.ELEMENT:
            return [value]

        # choose finding method
        if timeout > 0:
            get = slow
        else:
            get = fast

        # find elements
        return get((by, value))

    def get_elements_text(self, locator, timeout=0):
        """ get text from elements
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[str]
        """
        return [element.text for element in self.find_all(locator, timeout)]

    def try_to_find(self, locator: tuple, timeout=0, index=0):
        """try to find element
        :param index: parameter for special cases
                      where a list of elements is in the locator,
                      the default element to return is elements[0]
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: WebElement | None"""
        try:
            return self.find(locator, timeout, index)
        except:
            return None

    def try_to_find_all(self, locator: tuple, timeout=0):
        """try to find elements
        :param timeout: how long to search
        :param locator: locate by method like id and value
        :rtype: list[WebElement]"""
        try:
            return self.find_all(locator, timeout)
        except:
            return []

    def try_to_get_elements_text(self, locator, timeout=0):
        """ get text from elements if any elements are located
        :type locator: tuple(str, str | list[WebElement] | WebElement)
        :type timeout: int
        :rtype: list[str] | []
        """
        element_list, elements_text = self.try_to_find_all(locator, timeout), []
        for i in range(len(element_list)):
            try:
                elements_text.append(element_list[i].text)
            except:
                pass
        return elements_text



import errno
import ntpath
import os
import pathlib
import shutil
import zipfile
from fnmatch import fnmatch, filter
from io import BytesIO
import imageio
from qaviton.utils.operating_system import s
import yaml
import glob


#############################################################################
#                                                                           #
#                              Directories                                  #
#                                                                           #
#############################################################################


def find_replace(directory, find, replace, pattern):
    """replace file patterns
    search for files with certain name patterns
    and find & replace a pattern inside the files"""
    for path, dirs, files in os.walk(os.path.abspath(directory)):
        for filename in filter(files, pattern):
            filepath = os.path.join(path, filename)
            with open(filepath) as f:
                s = f.read()
            s = s.replace(find, replace)
            with open(filepath, "w") as f:
                f.write(s)


def copy_directory(src, dest):
    "dest should be a directory path that does not exist"
    try:
        shutil.copytree(src, dest)
    except OSError as e:
        # If the error was caused because the source wasn't a directory
        if e.errno == errno.ENOTDIR:
            shutil.copy(src, dest)
        else:
            raise Exception('Directory not copied') from e


def get_directory_from_file(__file__: str):
    return os.path.dirname(os.path.abspath(__file__))


def create_directory(directory_path: str):
    if not os.path.exists(directory_path):
        try:
            os.makedirs(directory_path, exist_ok=True)
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
        except Exception as e:
            if not os.path.exists(directory_path):
                raise e


def create_directory_from_file(file_path: str):
    if not os.path.exists(file_path):
        try:
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
        except Exception as e:
            if not os.path.exists(os.path.dirname(file_path)):
                raise e


def get_subdirectories(directory_path: str):
    """ example:
    default_products_directory = settings.project_directory + 'products' + s
    default_products_directories = file_handler.get_subdirectories(default_products_directory)
    """
    for i in os.walk(directory_path):
        results = []
        for subdir_name in i[1]:
            results.append(directory_path + s + subdir_name)
        return results


def get_subdirectories_deep(directory_path: str):
    """get all subdirectories under root dir"""
    return [x[0] for x in os.walk(directory_path)]


def clean_directory(root_directory: str, pattern: str):
    """delete all files with the pattern under root dir"""
    files = deep_files_search(root_directory, '*' + pattern)
    for file in files:
        if pattern in str(file):
            if os.path.isfile(file):
                os.unlink(file)


def delete_directory_contents(root_directory: str):
    """delete directory and all its contents

    if this does not work because of permission error consider using:
        windows: filer.os.system('rd /s /q {}'.format(root_directory))
        linux/mac: filer.os.system('rm -rf {}'.format(root_directory))

        example:
            delete_directory_contents(test_report_directory_path)
    """
    for the_file in os.listdir(root_directory):
        file_path = os.path.join(root_directory, the_file)
        if os.path.isfile(file_path):
            os.unlink(file_path)
        elif os.path.isdir(file_path):
            shutil.rmtree(file_path)


def delete_directory(directory):
    """delete directory and all its contents
    preferably use delete_directory_contents function instead
    """
    if s == '\\':
        os.system('rd /s /q {}'.format(directory))
    else:
        os.system('rm -rf {}'.format(directory))
    if os.path.exists(directory):
        raise Exception("directory could not be deleted")


#############################################################################
#                                                                           #
#                                  Files                                    #
#                                                                           #
#############################################################################


def deep_files_search(root_directory, pattern="*.txt"):
    """
    search for files under a directory and all its subdirectories
    :param root_directory: /path/to/root/dir
    :param pattern: find_me.example
    :return: list of files paths
    """
    results = []
    for path, subdirs, files in os.walk(root_directory):
        for name in files:
            if fnmatch(name, pattern):
                results.append(str(pathlib.PurePath(path, name)))
    return results


def get_dir_files(dir_path):
    return [f for f in os.listdir(dir_path) if os.path.isfile(os.path.join(dir_path, f))]


def rename_file(name, new_name):
    os.rename(name, new_name)


def delete_file(name):
    os.remove(name)


def delete_all_files(dir):
    files = glob.glob(dir + s + '*')
    for f in files:
        os.remove(f)


def get_file_full_name(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


def get_file_name_and_type(file_full_name):
    return file_full_name.split('.')


def create_file(file_path: str):
    try:
        open(file_path, 'w+').close()
    except Exception as e:
        if not os.path.exists(file_path):
            raise e


def get_file_name(file_path):
    return os.path.basename(file_path)


def get_file_directory(path):
    return os.path.dirname(os.path.realpath(path))


def copy_file(src_file, dest_file):
    """use this in cases where another program
    might try to copy to same dest_file
    """
    if not os.path.exists(dest_file):
        try:
            shutil.copyfile(src_file, dest_file)
        except Exception as e:
            if not os.path.exists(dest_file):
                raise e


#############################################################################
#                                                                           #
#                                 Images                                    #
#                                                                           #
#############################################################################


def create_gif(image_files, gif_file_path):
    """
    create gif from png files
    :param image_files: list of png file paths
            example:
                image_files = [
                    "/path/to/your/png/file1.png",
                    "/path/to/your/png/file2.png",
                    "/path/to/your/png/file3.png"
                ]
    :param gif_file_path: /path/to/your/gif/file.gif
    :return:
    """
    with imageio.get_writer(gif_file_path, mode='I') as writer:
        for file in image_files:
            image = imageio.imread(file)
            writer.append_data(image)


#############################################################################
#                                                                           #
#                                  ZIP                                      #
#                                                                           #
#############################################################################


def read_zipfile(zip_obj, name):
    """
    :param zip_obj: zip file object
    :param name: name of file inside the zip
            example:
                myzip.zip
                    'file.txt'
    :return:
    """
    with zip_obj.open(name) as f:
        return f.read()


def read_zipfiles(zip_obj):
    """
    :param zip_obj: zip file object
    :return:
    """
    zip_files = []
    for name in zip_obj.namelist():
        zip_files.append(read_zipfile(zip_obj, name))
    return zip_files


def open_zip(zip_path):
    return zipfile.ZipFile(zip_path)


def open_zip_from_inside_zip(zip_obj, zfile_name):
    zfiledata = BytesIO(zip_obj.read(zfile_name))
    return zipfile.ZipFile(zfiledata)


#############################################################################
#                                                                           #
#                                 YAML                                      #
#                                                                           #
#############################################################################


class parse:
    @staticmethod
    def yaml(file_path: str):
        with open(file_path) as y:
            return yaml.load(y.read())


class dump:
    @staticmethod
    def yaml(dump_to_file_path: str, parsed_yaml: dict):
        with open(dump_to_file_path, 'w') as y:
            y.write(yaml.dump(parsed_yaml, default_flow_style=False))


from qaviton.utils.log_handler import get_logger, log_levels


def testlogger():
    log1 = get_logger('log1.txt', log_name='log1')
    log1.debug('debug msg')
    log1.info('info msg')
    log1.warning('warning msg')
    log1.error('error msg')
    log1.critical('critical msg')

    log2 = get_logger('log2.txt', log_name='log2', log_level=log_levels.INFO, log_to_console=False)
    log2.debug('debug msg')
    log2.info('info msg')
    log2.warning('warning msg')
    log2.error('error msg')
    log2.critical('critical msg')



from tests.pages.components.page import Page
from tests.config.locators import locator


class QavitonHomePage(Page):
    def SIGN_UP_FOR_A_BETA(self):
        return self.find(locator.SIGN_UP_FOR_A_BETA)

    def qaviton_menu_home_button(self):
        return self.find(locator.qaviton_menu_home_button)

    def qaviton_logo(self):
        return self.find(locator.qaviton_logo)

    def qaviton_send_demo_request(self):
        return self.find(locator.qaviton_send_demo_request)

    def qaviton_name_demo_request(self):
        return self.find(locator.qaviton_name_demo_request)

    def qaviton_company_demo_request(self):
        return self.find(locator.qaviton_company_demo_request)

    def qaviton_email_demo_request(self):
        return self.find(locator.qaviton_email_demo_request)



from qaviton.navigator import Navigator
from tests.pages.components.page import Page
from tests.pages.qaviton_home import QavitonHomePage
from tests.pages.linkedin_home import LinkedinHomePage
from tests.pages.google_search import GoogleSearchPage
from tests.pages.google_home import GoogleHomePage


class App(Page):
    """use the app page to include all your
    pages/components/api/services/flows
    in a single page application
    """
    def __init__(self, driver, platform):
        Page.__init__(self, driver, platform=platform)
        self.qaviton_home = QavitonHomePage(driver)
        self.linkedin_home = LinkedinHomePage(driver)
        self.google_search = GoogleSearchPage(driver)
        self.google_home = GoogleHomePage(driver)

        self.navigate = Navigator(self.qaviton_home, auto_connect=self)



from qaviton.navigator import Graph


def test_graph():
    # build graph
    graph = Graph()

    # add nodes
    graph.add_nodes({'S', 'a', 'b', 'c', 'd', 'e', 'f', 'h', 'p', 'q', 'r', 'G'})

    # describe node connections
    graph.connect('S', 'd', 'S -> d', 3)
    graph.connect('S', 'e', 'S -> e', 9)
    graph.connect('S', 'p', 'S -> p', 1)
    graph.connect('b', 'a', 'b -> a', 2)
    graph.connect('c', 'a', 'c -> a', 2)
    graph.connect('d', 'b', 'd -> b', 1)
    graph.connect('d', 'c', 'd -> c', 8)
    graph.connect('d', 'e', 'd -> e', 2)
    graph.connect('e', 'h', 'e -> h', 8)
    graph.connect('e', 'r', 'e -> r', 2)
    graph.connect('f', 'c', 'f -> c', 3)
    graph.connect('f', 'G', 'f -> G', 2)
    graph.connect('h', 'p', 'h -> p', 4)
    graph.connect('h', 'q', 'h -> q', 4)
    graph.connect('p', 'q', 'p -> q', 15)
    graph.connect('r', 'f', 'r -> f', 1)

    # find shortest path
    cost, path, nodes = graph.find_path(start='S', goal='G')
    assert cost == 10
    assert str(path) == "['S -> d', 'd -> e', 'e -> r', 'r -> f', 'f -> G']"
    expected_nodes = ('d', 'e', 'r', 'f', 'G')
    for i in range(len(expected_nodes)):
        assert expected_nodes[i] == nodes[i].object



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


""" updating pip version:
    ---------------------

1) increase qaviton.version.__version__

2) make sure these are installed:
    pip install wheel
    python setup.py bdist_wheel --universal
    pip install twine

3) run these:
    python setup.py sdist
    python setup.py bdist_wheel
    twine upload dist/*
"""


import io
from qaviton.version import __version__ as version
from setuptools import setup, find_packages


packages = []
for package in find_packages():
    if package.startswith('qaviton'):
        packages.append(package)


with io.open('README.rst', 'rt', encoding='utf8') as f:
    readme = f.read()


setup(
    name='qaviton',
    version=version,
    description='python implementation of qaviton',
    long_description=readme,
    keywords=['qaviton'],
    author='Yehonadav Bar Elan',
    author_email='yehonadav@Qaviton.com',
    url='https://qaviton.com/',
    packages=packages,
    data_files=[('qaviton', ['qaviton/steps.json'])],
    include_package_data=True,
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Environment :: Console',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    install_requires=[
        'Appium-Python-Client>=0.28',
        'selenium>=3.14.0',
        'Faker>=0.8.17',
        'keyboard>=0.13.2',
        'mouse>=0.7.0',
        'imageio>=2.3.0',
        'pytest>=3.7.1',
        'pytest-xdist>=1.22.5',
        'pytest-cov>=2.5.1',
        'pytest-ordering>=0.5',
        'pytest-forked>=0.2',
        'threaders>=0.2.13',
        'PyYAML>=3.13',
        'requests>=2.19.1'
    ]
)



import pytest
from qaviton.utils import filer
from qaviton.utils import condition
from tests.config.file_paths import yaml1
from tests.config.file_paths import yaml2


@pytest.fixture
def parsed_yaml_file():
    yield filer.parse.yaml(yaml1)


def test_yaml1(parsed_yaml_file):
    y = parsed_yaml_file
    assert condition.all_in_all(("a", "b"), y.keys())
    assert y["a"] == 1
    assert isinstance(y["b"], dict)
    assert condition.all_in_all(("c", "d"), y["b"].keys())
    assert y["b"]["c"] == 3
    assert y["b"]["d"] == 4


def test_yaml2(parsed_yaml_file):
    y = parsed_yaml_file
    doc = """a: 1
b:
  c: 3
  d: 4
"""
    filer.dump.yaml(yaml2, y)
    with open(yaml2) as f:
        assert f.read() == doc



# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""The WebDriver implementation."""

from time import sleep
from selenium.webdriver import Remote
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.wait import POLL_FREQUENCY
from qaviton.drivers.common.webdriver import WebDriver as WE
from qaviton.drivers.common.webelement import WebElement


class WebDriver(WE, Remote):
    """
    Controls a browser by sending commands to a remote server.
    This server is expected to be running the WebDriver wire protocol
    as defined at
    https://github.com/SeleniumHQ/selenium/wiki/JsonWireProtocol

    :Attributes:
     - session_id - String ID of the browser session started and controlled by this WebDriver.
     - capabilities - Dictionaty of effective capabilities of this browser session as returned
         by the remote server. See https://github.com/SeleniumHQ/selenium/wiki/DesiredCapabilities
     - command_executor - remote_connection.RemoteConnection object used to execute commands.
     - error_handler - errorhandler.ErrorHandler object used to handle errors.
    """

    def __init__(self, command_executor='http://127.0.0.1:4444/wd/hub',
                 desired_capabilities=None, browser_profile=None, proxy=None,
                 keep_alive=False, file_detector=None, options=None):

        Remote.__init__(self, command_executor, desired_capabilities, browser_profile,
                        proxy, keep_alive, file_detector, options)

    def create_web_element(self, element_id):
        """
        Creates a web element with the specified element_id.
        Overrides method and appium Selenium WebDriver in order to always create the qaviton extended WebElement
        """
        return WebElement(self, element_id)

    def zoom(self, percent=200, *args, **kwargs):
        """ zoom like appium for selenium (will not work for all browsers)
            Zooms in on an element a certain amount

                :Args:
                 - percent - (optional) amount to zoom. Defaults to 200%
                 - args/kwargs - catch any unwanted arguments that may come from appium zoom

                :Usage:
                    driver.zoom(150)
                    driver.zoom(50)
                    driver.zoom(100)
                """
        self.execute_script("document.body.style.zoom = arguments[0]+'%';", str(percent))
        return self

    def drag_and_drop(self, origin_el, destination_el):
        """Drag the origin element to the destination element

        :Args:
         - originEl - the element to drag
         - destinationEl - the element to drag to
        """
        ActionChains(self).drag_and_drop(origin_el, destination_el).perform()
        return self

    def scroll_element_into_view(self, element, retries=4):
        """ selenium only: scrolling function to scroll until element is visible (whole window scroll)
        :type element: WebElement
        :param retries: integer bigger or equal to 0
        :rtype: WebElement
        """
        for retry in range(1+retries):
            try:
                self.execute_script("arguments[0].scrollIntoView(true);", element)
                return element
            except Exception as e:
                if retry == retries:
                    raise e
                sleep(POLL_FREQUENCY)
        return element

    def get_attributes(self, element):
        """ get all the attributes
        :rtype: dict
        """
        return self.execute_script(
            "var items = {}; "
            "for (index = 0; index < arguments[0].attributes.length; ++index) { "
            "items[arguments[0].attributes[index].name] = arguments[0].attributes[index].value }; "
            "return items;",
            element)



import threading


def test_app1(app):
    app.navigate(app.linkedin_home).register.register_and_submit()


def test_app2(app, app2):
    app.navigate(app.linkedin_home).register.register_and_submit()
    app2.navigate(app2.linkedin_home).register.register_and_submit()


def test_app3(app):
    app.navigate(app.qaviton_home).SIGN_UP_FOR_A_BETA()
    app.qaviton_home.qaviton_menu_home_button()
    app.qaviton_home.qaviton_logo()
    app.qaviton_home.qaviton_send_demo_request()
    app.qaviton_home.qaviton_name_demo_request()
    app.qaviton_home.qaviton_company_demo_request()
    app.qaviton_home.qaviton_email_demo_request()


def test_app4(app, app2):
    t1 = threading.Thread(target=lambda: app.navigate(app.google_search).search_bar.search('qaviton'))
    t2 = threading.Thread(target=lambda: app2.navigate(app2.google_search).search_bar.search('cats'))
    t1.start(), t2.start()
    t1.join(), t2.join()


def test_app5(app):
    app.navigate(app.linkedin_home)


def test_app6(app):
    app.navigate(app.google_home).google_search_button()


def test_app7(app, app2):
    app.navigate(app.google_search).search_bar.button()
    app2.navigate(app2.google_search).search_bar.button()



import pytest


def dependencies(parametrize, names=None, depends=None, xfail=None, xfail_reason=''):
    r = []
    if xfail is None:
        if names is None and depends is None:
            for param in parametrize:
                r.append(pytest.mark.dependency()(param))

        elif names is not None and depends is None:
            for i in range(len(parametrize)):
                r.append(pytest.mark.dependency(name=names[i])(parametrize[i]))

        elif names is None and depends is not None:
            for param in parametrize:
                r.append(pytest.mark.dependency(depends=depends)(param))

        else:
            for i in range(len(parametrize)):
                r.append(pytest.mark.dependency(name=names[i], depends=depends)(parametrize[i]))
        return r

    elif xfail is True:
        if names is None and depends is None:
            for param in parametrize:
                r.append(pytest.mark.dependency()(pytest.mark.xfail(param, reason=xfail_reason)))

        elif names is not None and depends is None:
            for i in range(len(parametrize)):
                r.append(pytest.mark.dependency(name=names[i])(pytest.mark.xfail(parametrize[i], reason=xfail_reason)))

        elif names is None and depends is not None:
            for param in parametrize:
                r.append(pytest.mark.dependency(depends=depends)(pytest.mark.xfail(param, reason=xfail_reason)))

        else:
            for i in range(len(parametrize)):
                r.append(pytest.mark.dependency(name=names[i], depends=depends)(
                    pytest.mark.xfail(parametrize[i], reason=xfail_reason)))
        return r

    else:
        if names is None and depends is None:
            for i in range(len(parametrize)):
                if i in xfail:
                    r.append(pytest.mark.dependency()(pytest.mark.xfail(parametrize[i], reason=xfail_reason)))
                else:
                    r.append(pytest.mark.dependency()(parametrize[i]))

        elif names is not None and depends is None:
            for i in range(len(parametrize)):
                if i in xfail:
                    r.append(
                        pytest.mark.dependency(name=names[i])(pytest.mark.xfail(parametrize[i], reason=xfail_reason)))
                else:
                    r.append(pytest.mark.dependency(name=names[i])(parametrize[i]))

        elif names is None and depends is not None:
            for i in range(len(parametrize)):
                if i in xfail:
                    r.append(
                        pytest.mark.dependency(depends=depends)(pytest.mark.xfail(parametrize[i], reason=xfail_reason)))
                else:
                    r.append(pytest.mark.dependency(depends=depends)(parametrize[i]))

        else:
            for i in range(len(parametrize)):
                if i in xfail:
                    r.append(pytest.mark.dependency(name=names[i], depends=depends)(
                        pytest.mark.xfail(parametrize[i], reason=xfail_reason)))
                else:
                    r.append(pytest.mark.dependency(name=names[i], depends=depends)(parametrize[i]))
        return r